"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getConfigExports = void 0;
const core_1 = require("@babel/core");
const fs_1 = require("fs");
const path_1 = require("path");
const asserts_1 = require("./asserts");
const resolveModule_1 = require("./resolveModule");
function toRelativeSource(filePath) {
    const pathChunks = filePath.split(path_1.sep);
    const relativeChunks = pathChunks.slice(pathChunks.lastIndexOf('node_modules') + 1);
    return relativeChunks.join('/');
}
function resolveIndexPath(name, indexFile) {
    if (!indexFile) {
        const indexFilePath = resolveModule_1.resolveModule(name);
        asserts_1.assertNotNull(indexFilePath, "failed to find 'indexFile' of '%s'.", name);
        return indexFilePath;
    }
    const pkgJSON = resolveModule_1.resolveModule(`${name}/package.json`);
    asserts_1.assertNotNull(pkgJSON, "failed to resolve package.json for '%s'", name);
    const pkgDir = path_1.dirname(pkgJSON);
    const indexFilePath = resolveModule_1.resolveModule(path_1.join(pkgDir, indexFile));
    asserts_1.assertNotNull(indexFilePath, "failed to resolve 'indexFile' specified for '%s'.", name);
    return indexFilePath;
}
function fulfillExports(exports, filePath) {
    const content = fs_1.readFileSync(filePath, 'utf-8');
    const ast = core_1.parse(content, { filename: filePath, sourceType: 'module' });
    const program = !ast ? null : core_1.types.isFile(ast) ? ast.program : ast;
    asserts_1.assertNotNull(program, "failed to parse '%s'.", filePath);
    const fileDir = path_1.dirname(filePath);
    const imports = new Map();
    for (const node of program.body) {
        if (core_1.types.isImportDeclaration(node)) {
            const sourcePath = resolveModule_1.resolveModule(node.source.value, fileDir);
            if (!sourcePath) {
                continue;
            }
            for (const specifier of node.specifiers) {
                imports.set(specifier.local.name, {
                    source: toRelativeSource(sourcePath),
                    imported: core_1.types.isImportNamespaceSpecifier(specifier)
                        ? '*'
                        : core_1.types.isImportDefaultSpecifier(specifier)
                            ? 'default'
                            : core_1.types.isIdentifier(specifier.imported)
                                ? specifier.imported.name
                                : specifier.imported.value,
                });
            }
        }
        else if (core_1.types.isExportNamedDeclaration(node)) {
            const nodeDeclaration = node.declaration;
            if (core_1.types.isVariableDeclaration(nodeDeclaration)) {
                for (const declaration of nodeDeclaration.declarations) {
                    if (core_1.types.isIdentifier(declaration.id) &&
                        core_1.types.isIdentifier(declaration.init)) {
                        const external = declaration.id.name;
                        const internal = declaration.init.name;
                        const imported = imports.get(internal);
                        if (imported) {
                            exports.set(external, {
                                external,
                                source: imported.source,
                                internal: imported.imported,
                            });
                        }
                    }
                }
            }
            const sourcePath = !node.source
                ? null
                : resolveModule_1.resolveModule(node.source.value, fileDir);
            for (const specifier of node.specifiers) {
                if (core_1.types.isExportSpecifier(specifier)) {
                    const internal = specifier.local.name;
                    const external = core_1.types.isIdentifier(specifier.exported)
                        ? specifier.exported.name
                        : specifier.exported.value;
                    if (sourcePath) {
                        exports.set(external, {
                            internal,
                            external,
                            source: toRelativeSource(sourcePath),
                        });
                    }
                    else {
                        const imported = imports.get(internal);
                        if (imported) {
                            exports.set(external, {
                                external,
                                source: imported.source,
                                internal: imported.imported,
                            });
                        }
                    }
                }
            }
        }
        else if (core_1.types.isExportAllDeclaration(node)) {
            const sourcePath = resolveModule_1.resolveModule(node.source.value, fileDir);
            asserts_1.assertNotNull(sourcePath, 'failed to resolve %s', sourcePath);
            fulfillExports(exports, sourcePath);
        }
    }
}
function getConfigExports(config) {
    const indexPath = resolveIndexPath(config.name, config.indexFile);
    const exports = new Map();
    fulfillExports(exports, indexPath);
    return exports;
}
exports.getConfigExports = getConfigExports;
